import time
from calendar import monthrange, timegm
from datetime import datetime, timedelta


class Date(object):
    """
    Class for encapsulating date objects.
    """

    UNSET = -1


    def __init__(self, year, month, day, hours = -1, mins = -1, secs = -1,
                 is_utc = False):

        self.__year = year
        self.__month = month
        self.__day = day
        self.__hours = hours
        self.__mins = mins
        self.__secs = secs
        self.__month_overflow_day = 0
        

        if (is_utc): self.__utc_to_localtime()

        # Validate the time
        try:
            sttime = self._to_time()
        except ValueError:
            print "ERROR: This date does not exist: " \
                  "%s %s %s %s:%s:%s" % (year, month, day, hours, mins, secs)


    def __cmp__(self, other):

        return cmp(self.get_day(), other.get_day())



    def __repr__(self):

        return str(self.get_day()) + str(self.get_daytime())



    def __hash__(self):

        return hash(repr(self))




    def _to_time(self):
        """
        Returns the time tuple of this date.
        """
        
        year, month, day = self.__year, self.__month, self.__day
        h = self.__hours
        m = self.__mins
        s = self.__secs

        if (self.has_daytime()):
            value = time.strptime("%(year)d %(month)d %(day)d "
                                  "%(h)d %(m)d %(s)d" % vars(),
                                  "%Y %m %d %H %M %S")
        else:
            value = time.strptime("%(year)d %(month)d %(day)d" % vars(),
                                  "%Y %m %d")
            
        return value


    def __utc_to_localtime(self):
        """
        Converts this UTC date to local time.
        """

        # get time difference
        secs = timegm(self._to_time())
        localtime = time.localtime(secs)
        localsecs = timegm(localtime)
        diff = int(localsecs - secs)

        # add difference
        self.add_time(0, 0, 0, 0, 0, diff)



    def copy(self):
        """
        Clones this Date object.
        """
        
        clone = Date(self.__year, self.__month, self.__day,
                     self.__hours, self.__mins, self.__secs)
        return clone



    def get_day(self):
        """
        Returns the day tuple (y, m, d) for this date.
        """

        y, m, d = self.__year, self.__month, self.__day
        return (y, m, d)


    def get_daytime(self):
        """
        Returns the day time tuple (h, m, s) for this date.
        """

        return (self.__hours, self.__mins, self.__secs)


    def has_daytime(self):
        """
        Returns whether the daytime is set-
        """

        return (self.__hours != self.UNSET)



    def get_interval(self, other):
        """
        Returns the time interval to another date in seconds.
        """

        t1 = self._to_time()
        t2 = other._to_time()

        diff = time.mktime(t2) - time.mktime(t1)

        return int(diff)


    def add_time(self, dyear, dmonth, dday, dhour = 0, dmin = 0, dsec = 0):
        """
        Adds a given amount of time to this date. The amount may be negative.
        """
        
        # handle month day overflow
        if (self.__month_overflow_day):
            self.__day = self.__month_overflow_day
            self.__month_overflow_day = 0
        
        self.__month += dmonth
        while (self.__month > 12):
            self.__month -= 12
            self.__year += 1
        self.__year += dyear

        # detect month day overflow
        (firstday, numdays) = monthrange(self.__year, self.__month)
        if (self.__day > numdays):
            self.__month_overflow_day = self.__day
            self.__day = numdays


        # add time
        tsecs = time.mktime(self._to_time())
        tsecs += dday * 24 * 3600 + dhour * 3600 + dmin * 60 + dsec
        t = time.localtime(tsecs)
        
        self.__year = t[0]
        self.__month = t[1]
        self.__day =   t[2]
        self.__hours = t[3]
        self.__mins =  t[4]
        self.__secs =  t[5]
